function [A, T, N] = estload( fname );
%LOADESTFILE Load file in EST_Track format
%
% A = estload( FILENAME )
%
% Function to load data files of Edinburgh Speech Tools EST_Track
% format.  Returns the Track in matrix A, with time increasing
% down the rows, and each channel in its respective column.
%
% [A, T] = estload( FILENAME )
% Optionally returns the time track of file in Vector T
%
% [A, T, N] = estload( FILENAME )
% Optionally returns the channel names in cell array N
%
% NOTE: if the file FILENAME contains only time points and no
%       data (such as a pitchmark file), then matrix A will be 
%       empty.
%
% See also ESTPARSE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%    Author: Korin Richmond                                    %%
%%   Purpose: Function for loading EST Track files into matlab  %%
%%   Created: Fri Feb 11 16:10:04 2000                          %%
%%                                                              %%
%%  Copyright (C) Korin Richmond 2000                           %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% Licence conditions apply to the download and use of this     %%
%% software - please see the file "LICENCE" accompanying this   %%
%% file in the same directory for details                       %%
%%                                                              %%
%% DISCLAIMER:                                                  %%
%%                                                              %%
%% EDINBURGH UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK       %%
%% DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,        %%
%% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND      %%
%% FITNESS. IN NO EVENT SHALL EDINBURGH UNIVERSITY OR THE       %%
%% CONTRIBUTORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR          %%
%% CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING    %%
%% FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF   %%
%% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT   %%
%% OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS      %% 
%% SOFTWARE.                                                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% no need to check file existence - estparse will do it
[D, dtype, nrows, ncols, bo, N] = estparse( fname );

[fid, msg] = fopen( fname, 'r', bo );
fseek( fid, D, 'bof' ); 

if strcmp(dtype, 'binary')
  A=fread( fid, [ncols+2,Inf], 'float32' );
  fclose(fid);
elseif strcmp(dtype, 'ascii')
  [A, count]=fscanf( fid, '%f', [ncols+2, nrows] );
  fclose(fid);
  
  if count/(ncols+2) ~= nrows
    mesg=sprintf('rows read (%d) != nrows in file header (%d)', count, nrows); 
    error( mesg );
  end
end

T=A(1,:)';

if( ncols == 0 )
  A = [];
else
  delete_indx = size(A, 1) - ncols;
  A(1:delete_indx,:) = [];
  A=A';
end
