function estsave( F, M, T, format, N )
%function ESTSAVE( F, M, T, format, N )
%
% Function to save Matrices in EST Track file format
%
%  where, 
%  F is string containing file name (including path)
%  M is Matrix/Vector of values to save
%  T is Vector of time values
%  format is either 'ascii' or 'binary' (default)
%  N is 1D cell array of channel names (optional)
%
% See also ESTLOAD, ESTPARSE

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%    Author: Korin Richmond                                    %%
%%   Purpose: Function for loading EST Track files into matlab  %%
%%   Created:  11 March 2001                                    %%
%%                                                              %%
%%  Copyright (C) Korin Richmond 2001                           %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                              %%
%% Licence conditions apply to the download and use of this     %%
%% software - please see the file "LICENCE" accompanying this   %%
%% file in the same directory for details                       %%
%%                                                              %%
%% DISCLAIMER:                                                  %%
%%                                                              %%
%% EDINBURGH UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK       %%
%% DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,        %%
%% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND      %%
%% FITNESS. IN NO EVENT SHALL EDINBURGH UNIVERSITY OR THE       %%
%% CONTRIBUTORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR          %%
%% CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING    %%
%% FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF   %%
%% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT   %%
%% OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS      %% 
%% SOFTWARE.                                                    %%
%%                                                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% default to binary format
if nargin < 3 || strcmp( format, 'ascii')
  binaryfmt = 0;
else
  binaryfmt = 1;
end

[fid, mesg]=fopen( F, 'w', 'ieee-le' );

if fid < 0
  error( ['Error opening file: ' F ' ' mesg] );
end

fprintf( fid, 'EST_File Track\n' ); % EST header identifier  
if binaryfmt == 1
  fprintf( fid, 'DataType binary\n' );
else
  fprintf( fid, 'DataType ascii\n' );
end

fprintf( fid, 'NumFrames %d\n', size(M,1) );
fprintf( fid, 'NumChannels %d\n', size(M,2) );
fprintf( fid, 'NumAuxChannels 0\n' );
fprintf( fid, 'EqualSpace 0\n' ); % might not be true, but this is (suboptimal) simplification 
fprintf( fid, 'BreaksPresent true\n' ); % might not be true, but this is (suboptimal) simplification 

if nargin == 5 
  for i=1:size(M,2)
    fprintf( fid, 'Channel_%d %s\n', i-1, N{i} );
  end
else
  for i=1:size(M,2)
    fprintf( fid, 'Channel_%d track_%d\n', i-1, i-1 );
  end
end

fprintf( fid, 'ByteOrder 01\n' ); % little endian
fprintf( fid, 'CommentChar ;\n' );
fprintf( fid, 'name %s\n', F );
fprintf( fid, 'EST_Header_End\n' );


if binaryfmt == 0
  if nargin <= 2
    for i=1:size(M,1)
      fprintf( fid, '%f\t', i );
      fprintf( fid, '1\t' );
      fprintf( fid, '%f ', M(i,:) );
      fprintf( fid, '\n' );
    end
  else
    for i=1:size(M,1)
      fprintf( fid, '%f\t', T(i) );
      fprintf( fid, '1\t' );
      fprintf( fid, '%f ', M(i,:) );
      fprintf( fid, '\n' );
    end
  end
else
  fwrite( fid, [T ones(length(T),1) M]', 'float32' );
end

fclose(fid);

