function nn = nnapplygrads(nn, j, epoch)
%NNAPPLYGRADS updates weights and biases with calculated gradients
% nn = nnapplygrads(nn) returns an neural network structure with updated
% weights and biases

    % Applies back-propagation only to the output layer for N epochs, where
    % N is given by i < N+1
    if j < 1
        temp = nn.n-1;
        if(nn.weightPenaltyL2>0)
            dW = nn.dW{temp} + nn.weightPenaltyL2 * ...
                [zeros(size(nn.W{temp},1),1) nn.W{temp}(:,2:end)];
        else
            dW = nn.dW{temp};
        end

        dW = nn.learningRate * dW;

        if(nn.momentum>0)
            nn.vW{temp} = nn.momentum*nn.vW{temp} + dW;
            dW = nn.vW{temp};
        end

        nn.W{temp} = nn.W{temp} - dW;
    else
        for i = 1 : (nn.n - 1)
            %The following varies the learning rate for the different
            %layers
            if i == 1
                nn.learningRate = 0.5;
            else
                nn.learningRate = 0.01;
            end
            if(nn.weightPenaltyL2>0)
                dW = nn.dW{i} + nn.weightPenaltyL2 * ...
                    [zeros(size(nn.W{i},1),1) nn.W{i}(:,2:end)];
            else
                dW = nn.dW{i};
            end

            dW = nn.learningRate * dW;

            if(nn.momentum>0)
                nn.vW{i} = nn.momentum*nn.vW{i} + dW;
                dW = nn.vW{i};
            end

            nn.W{i} = nn.W{i} - dW;
        end
    end
end
